# stolen from https://github.com/pyparsing/pyparsing/blob/master/examples/simpleSQL.py

from typing import Any, TypedDict

from pyparsing import (
    CaselessKeyword,
    Forward,
    Group,
    Optional,
    ParserElement,
    Suppress,
    Word,
    alphanums,
    alphas,
    delimitedList,
    infixNotation,
    oneOf,
    opAssoc,
    pyparsing_common,
    quotedString,
)

from python_soql_parser.binops import EQ, GT, GTE, LT, LTE, NEQ

ParserElement.enablePackrat()

select_statement = Forward()
SELECT, FROM, WHERE, AND, OR, IN, NULL, TRUE, FALSE, LIMIT, ORDER, BY, DESC, ASC = map(
    CaselessKeyword,
    "select from where and or in null true false limit order by desc asc".split(),
)

identifier = Word(alphas, alphanums + "_" + ".").setName("identifier")
field_name = delimitedList(identifier).setName("field name")
field_name_list = Group(delimitedList(field_name))
sobject_name = identifier.setName("sobject name")


binop = oneOf(f"{EQ} {NEQ} {LT} {LTE} {GT} {GTE}")
real_num = pyparsing_common.real()
int_num = pyparsing_common.signed_integer()

field_right_value = real_num | int_num | quotedString | field_name
where_condition = Group(
    (field_name + binop + field_right_value)
    | (field_name + IN + Group("(" + delimitedList(field_right_value) + ")"))
)

where_expression = infixNotation(
    where_condition,
    [
        (AND, 2, opAssoc.LEFT),
        (OR, 2, opAssoc.LEFT),
    ],
)

where_clause = Optional(Suppress(WHERE) + where_expression, None)

limit_clause = Optional(Suppress(LIMIT) + int_num, None)

ordering_term = Group(field_name + Optional(ASC | DESC)("direction"))

order_clause = Optional(
    Suppress(ORDER) + Suppress(BY) + Group(delimitedList(ordering_term))
)

# define the grammar
select_statement <<= (
    SELECT
    + field_name_list("fields")
    + FROM
    + sobject_name("sobject")
    + where_clause("where")
    + order_clause("order_by")
    + limit_clause("limit")
)

soql = select_statement


class SoqlQuery(TypedDict):
    # TODO: type the `Any`s
    fields: Any
    sobject: str
    where: Any
    limit: Any


def parse(soql_query: str) -> SoqlQuery:
    return soql.parseString(soql_query)



YESTERDAY = "YESTERDAY"
TODAY = "TODAY"
TOMORROW = "TOMORROW"
LAST_WEEK = "LAST_WEEK"
THIS_WEEK = "THIS_WEEK"
NEXT_WEEK = "NEXT_WEEK"
LAST_MONTH = "LAST_MONTH"
THIS_MONTH = "THIS_MONTH"
NEXT_MONTH = "NEXT_MONTH"
LAST_90_DAYS = "LAST_90_DAYS"
NEXT_90_DAYS = "NEXT_90_DAYS"
LAST_N_DAYS = "LAST_N_DAYS"
NEXT_N_DAYS = "NEXT_N_DAYS"
NEXT_N_WEEKS = "NEXT_N_WEEKS"
LAST_N_WEEKS = "LAST_N_WEEKS"
NEXT_N_MONTHS = "NEXT_N_MONTHS"
LAST_N_MONTHS = "LAST_N_MONTHS"
THIS_QUARTER = "THIS_QUARTER"
LAST_QUARTER = "LAST_QUARTER"
NEXT_QUARTER = "NEXT_QUARTER"
NEXT_N_QUARTERS = "NEXT_N_QUARTERS"
LAST_N_QUARTERS = "LAST_N_QUARTERS"
THIS_YEAR = "THIS_YEAR"
LAST_YEAR = "LAST_YEAR"
NEXT_YEAR = "NEXT_YEAR"
NEXT_N_YEARS = "NEXT_N_YEARS"
LAST_N_YEARS = "LAST_N_YEARS"
THIS_FISCAL_QUARTER = "THIS_FISCAL_QUARTER"
LAST_FISCAL_QUARTER = "LAST_FISCAL_QUARTER"
NEXT_FISCAL_QUARTER = "NEXT_FISCAL_QUARTER"
NEXT_N_FISCAL_QUARTERS = "NEXT_N_FISCAL_​QUARTERS"
LAST_N_FISCAL_QUARTERS = "LAST_N_FISCAL_​QUARTERS"
THIS_FISCAL_YEAR = "THIS_FISCAL_YEAR"
LAST_FISCAL_YEAR = "LAST_FISCAL_YEAR"
NEXT_FISCAL_YEAR = "NEXT_FISCAL_YEAR"
NEXT_N_FISCAL_YEARS = "NEXT_N_FISCAL_YEARS"
LAST_N_FISCAL_YEARS = "LAST_N_FISCAL_YEARS"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['python_soql_parser']

package_data = \
{'': ['*']}

install_requires = \
['pyparsing>=2.4.7,<3.0.0']

setup_kwargs = {
    'name': 'python-soql-parser',
    'version': '0.1.10',
    'description': 'A pyparsing-based library for parsing SOQL statements',
    'long_description': '**CONTRIBUTORS WANTED!!**\n\n# Installation\n\n`pip install python-soql-parser`\n\nor, with poetry\n\n`poetry add python-soql-parser`\n\n# Usage\n\n```python\nfrom python_soql_parser import parse\n\n\nparse_result = parse("SELECT Id FROM Account")\n```\n\nwhere `parse_result` is a [ParseResults](https://pyparsing-docs.readthedocs.io/en/latest/HowToUsePyparsing.html#parseresults) object from [pyparsing](https://github.com/pyparsing/pyparsing/).\n\n# Notable caveats\n\n## Unsupported features\n\n- Subqueries (e.g., `SELECT Name, (SELECT LastName FROM Contacts) FROM Account`)\n- Aggregate queries\n- SOQL specific WHERE-clause tokens (e.g., `LAST_N_DAYS:<integer>`)\n\n## Partially supported\n\nThese are either partially supported or weakly supported\n\n- Related attributes (e.g., `SELECT Id, Account.Name FROM Contact`). The object name is not yet parsed out on its own. This will simply parse out `"Account.Name"`\n\n# Contributing\n\nA lot of work remains to be done. Practically no SOQL-specific features are supported as of yet.\nIf you want to contribute, just open a PR! (and add a test for your new feature)\n\n## Setting up locally\n\nFirst install [poetry](https://python-poetry.org/). Afterwards, to install the dependencies, run\n\n```\npoetry install\n```\n\n## Running the tests\n\nSimply execute\n\n```\npytest\n```\n\n## House cleaning\n\nPlease sort imports with `isort` and format the code with `black` (in that order).\n',
    'author': 'Alex Drozd',
    'author_email': 'drozdster@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Kicksaw-Consulting/python-soql-parser',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
